import { Hit } from '@algolia/client-search';
import { ShoppingGuideType } from '@algolia/generative-experiences-api-client';

export declare type ChildrenProps = ComponentProps & {
    status: 'loading' | 'stalled' | 'idle';
    View(props: unknown): JSX.Element;
};

export declare type CommerceGetters = {
    /**
     * URL for a specific guide.
     */
    guideURL: (objectID: string) => string;
    /**
     * URL for a specific product.
     */
    objectURL: (objectID: string) => string;
    /**
     * List of images for a product.
     */
    images: (object: Record<string, unknown>) => Array<{
        src: string;
        alt: string;
    }>;
};

declare type ComponentChild = VNode<any> | object | string | number | boolean | null | undefined;

declare type ComponentChildren = ComponentChild[] | ComponentChild;

export declare type ComponentProps = {
    classNames: HeadlinesButtonClassNames;
    headlines: GSEHeadlineRecord[];
};

export declare type ContentChildrenProps = GSEContentComponentProps & {
    status: 'loading' | 'stalled' | 'idle';
    View(props: unknown): JSX.Element;
};

export declare type ContentClassNames = Partial<{
    wrapper?: string;
    container?: string;
    contentSection?: string;
    productLink?: string;
    heroImage?: string;
    factorSection?: string;
    relatedItemsSection?: string;
    relatedItemsListContainer?: string;
    relatedItemsTitle?: string;
    relatedItemsList?: string;
    item?: string;
}>;

export declare type ContentComponentProps = {
    itemComponent(props: ContentItemComponentProps): JSX.Element;
    item: GSEContentRecord;
    classNames?: ContentClassNames;
    children?(props: ContentChildrenProps): JSX.Element;
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
    alreadyCast?: boolean;
    showFeedback?: boolean;
    status: 'loading' | 'stalled' | 'idle';
    view?(props: ContentViewProps<Record<string, string>> & Renderer): JSX.Element;
    getters?: CommerceGetters;
};

export declare type ContentItemComponentProps = {
    hit: Hit;
    classNames?: ContentClassNames;
    getters: CommerceGetters;
} & Renderer;

export declare type ContentViewProps<TClassNames extends Record<string, string>> = {
    classNames: TClassNames;
    item: GSEContentRecord;
    itemComponent<TComponentProps extends Record<string, unknown> = {}>(props: {
        hit: Hit;
    } & Renderer & TComponentProps): JSX.Element;
    feedbackComponent<TComponentProps extends Record<string, unknown> = {}>(props: {
        castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
        alreadyCast?: boolean;
    } & Renderer & TComponentProps): JSX.Element;
    showFeedback?: boolean;
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
    alreadyCast?: boolean;
    getters: CommerceGetters;
};

export declare function createShoppingGuideContentComponent({ createElement, Fragment, }: Renderer): (props: ContentComponentProps) => JSX.Element;

export declare function createShoppingGuidesFeedbackComponent({ createElement, Fragment, }: Renderer): (props: FeedbackComponentProps) => JSX.Element;

export declare function createShoppingGuidesHeadlinesComponent({ createElement, Fragment, }: Renderer): (props: HeadlinesComponentProps) => JSX.Element;

export declare const defaultState: ShoppingGuideType;

export declare type FeedbackChildrenProps = {
    classNames?: FeedbackClassNames;
    alreadyCast?: boolean;
    status: 'loading' | 'stalled' | 'idle';
    View(props: unknown): JSX.Element;
};

export declare type FeedbackClassNames = Partial<{
    wrapper?: string;
    container?: string;
    button?: string;
    buttonIcon?: string;
    buttonsWrapper?: string;
    labelIcon?: string;
    labelWrapper?: string;
    noWrap?: string;
    screenReaderOnly?: string;
    thanksWrapper?: string;
}>;

export declare type FeedbackComponentProps = {
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
    alreadyCast?: boolean;
    classNames?: FeedbackClassNames;
    children?(props: FeedbackChildrenProps): JSX.Element;
    status: 'loading' | 'stalled' | 'idle';
    view?(props: FeedbackViewProps<Record<string, string>> & Renderer): JSX.Element;
};

export declare type FeedbackViewComponentProps = {
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
    alreadyCast?: boolean;
    classNames?: FeedbackClassNames;
    objectIDs?: string[];
    voteTarget?: 'content' | 'headline';
} & Renderer;

export declare type FeedbackViewProps<TClassNames extends Record<string, string>> = {
    classNames: TClassNames;
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
};

export declare type GSEContentComponentProps = {
    classNames?: ContentClassNames;
    content: GSEContentRecord;
};

export declare type GSEContentRecord = {
    objectID: string;
    status: 'draft' | 'generating' | 'headline' | 'published';
    title: string;
    score_content: number;
    generated_at: number;
} & ({
    type: 'category';
    description: string;
    category: string;
    objects: Hit[];
    content: Array<{
        type: 'conclusion' | 'factor' | 'introduction';
        title: string;
        content: string[];
    }>;
    score_headline: number;
} | {
    type: 'comparison';
    objects: Hit[];
    content: Array<{
        title: string;
        type: 'conclusion' | 'introduction' | 'product';
        objectID?: string;
        content: string[];
    }>;
    comparedObjectIDs: string[];
} | {
    type: 'shopping_guide';
    description: string;
    category: string;
    objects: Hit[];
    content: Array<{
        title: string;
        content: string[];
    }>;
    score_headline: number;
});

export declare type GSEHeadlineRecord = {
    objectID: string;
    title: string;
    description: string;
    objects: any[];
};

export declare type HeadlinesButtonClassNames = Partial<{
    wrapper?: string;
    container?: string;
    item?: string;
    itemContent?: string;
    itemTitle?: string;
    itemImage?: string;
    list?: string;
    readMore?: string;
}>;

export declare type HeadlinesComponentProps = {
    itemComponent?(props: ItemComponentProps): JSX.Element;
    items: GSEHeadlineRecord[];
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
    alreadyCast?: boolean;
    showFeedback?: boolean;
    classNames?: HeadlinesButtonClassNames;
    children?(props: ChildrenProps): JSX.Element;
    status: 'loading' | 'stalled' | 'idle';
    view?(props: ViewProps<GSEHeadlineRecord, Record<string, string>> & Renderer): JSX.Element;
    getters?: CommerceGetters;
};

export declare type ItemComponentProps = {
    item: GSEHeadlineRecord;
    classNames?: HeadlinesButtonClassNames;
    getters: CommerceGetters;
} & Renderer;

export declare type Pragma = (type: any, props: Record<string, any> | null, ...children: ComponentChildren[]) => JSX.Element;

export declare type PragmaFrag = any;

export declare type Renderer = {
    /**
     * The function to create virtual nodes.
     *
     * @default preact.createElement
     */
    createElement: Pragma;
    /**
     * The component to use to create fragments.
     *
     * @default preact.Fragment
     */
    Fragment: PragmaFrag;
};

export declare type ViewProps<TItem extends GSEHeadlineRecord, TClassNames extends Record<string, string>> = {
    classNames: TClassNames;
    itemComponent<TComponentProps extends Record<string, unknown> = {}>(props: {
        item: TItem;
    } & Renderer & TComponentProps): JSX.Element;
    feedbackComponent<TComponentProps extends Record<string, unknown> = {}>(props: {
        castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
        alreadyCast?: boolean;
    } & Renderer & TComponentProps): JSX.Element;
    items: TItem[];
    getters: CommerceGetters;
    showFeedback?: boolean;
    castFeedback: (voteType: 'upvote' | 'downvote', objectIDs?: string[], voteTarget?: 'content' | 'headline') => void;
    alreadyCast?: boolean;
};

export declare type VNode<TProps = any> = {
    type: any;
    key: string | number | any;
    props: TProps & {
        children: ComponentChildren;
        key?: any;
    };
};

export { }
